# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/lib/core/error_codes.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.tsl.protobuf import error_codes_pb2 as tensorflow_dot_tsl_dot_protobuf_dot_error__codes__pb2

from tensorflow.tsl.protobuf.error_codes_pb2 import *

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*tensorflow/core/lib/core/error_codes.proto\x1a)tensorflow/tsl/protobuf/error_codes.protoP\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.lib.core.error_codes_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
